# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
from abc import abstractmethod, ABCMeta

from autopilot.exceptions import ProcessSearchError
from psutil import NoSuchProcess
from retrying import retry
from subprocess import CalledProcessError

from ubuntu_system_tests.helpers import (
    autopilot,
    wait_until,
)
from ubuntu_system_tests.helpers import processes
from ubuntu_system_tests.helpers.scopes.apps import (
    launch_application_from_apps_scope)
from ubuntu_system_tests.helpers.unity8 import (
    launch_application_from_launcher)


class Application:
    __metaclass__ = ABCMeta

    def __init__(self, app_name, package, app_id=None):
        self.app_name = app_name
        self.app_package = package
        self.app_id = app_id
        self.proxy_object = None

    def launch_from_scope(self, return_proxy=True):
        """
        Launch app from apps scope
        """
        launch_application_from_apps_scope(self.app_name)
        self.wait_until_running()
        if return_proxy:
            return self.get_proxy_object()

    def launch_from_launcher(self, return_proxy=True):
        """Drag out the launcher and tap the application icon to start it."""
        launch_application_from_launcher(self.app_id)
        self.wait_until_running()
        if return_proxy:
            return self.get_proxy_object()

    def close(self):
        pass

    def stop(self):
        processes.stop_application(self.app_package)

    def get_pid(self):
        return processes.get_application_pid(self.app_package)

    def is_running(self):
        return processes.is_application_running(self.app_package)

    def wait_until_running(self):
        """Wait until the application is running."""
        if not wait_until(self.is_running):
            raise RuntimeError('Application not running.')

    @retry(stop_max_delay=30000,
           wait_exponential_multiplier=1000,
           wait_exponential_max=10000,
           retry_on_exception=lambda exception: (
               isinstance(exception, ProcessSearchError) or
               isinstance(exception, CalledProcessError) or
               isinstance(exception, processes.PidNotFoundError) or
               isinstance(exception, NoSuchProcess)))
    def get_proxy_object(self):
        if self.proxy_object is None:
            self.proxy_object = autopilot.get_proxy_object(
                self.get_pid())
        return self.proxy_object

    @abstractmethod
    def get_main_view(self):
        pass
