# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import subprocess

from ubuntu_system_tests.helpers.testbed import run_command_with_sudo
from ubuntu_system_tests.helpers.utils import wait_until


class JobBase:
    """This contains a base implementation for a Job entity and its
     common functions like:
     start, stop, get_status, etc.
     """

    def __init__(self, name, sudo=False):
        self.name = name
        self.start_command = '/sbin/initctl start {name}'.format(name=name)
        self.stop_command = '/sbin/initctl stop {name}'.format(name=name)
        self.status_command = '/sbin/initctl status {name}'.format(name=name)
        self.sudo = sudo

    def start(self, *args, **kwargs):
        """Start the job
        :param args: The arguments to be used when starting the job.
        """
        JobBase._run_command(self.start_command, self.sudo, *args, **kwargs)

    def stop(self, *args, **kwargs):
        """Stop the job"""
        JobBase._run_command(self.stop_command, self.sudo, *args, **kwargs)

    def restart(self, *args, **kwargs):
        """Restart the job"""
        self.ensure_stopped(*args, **kwargs)
        self.start(*args, **kwargs)

    def get_status(self, *args, **kwargs):
        """Gets the current status of the job"""
        return JobBase._run_command(self.status_command, self.sudo, *args,
                                    **kwargs)

    def get_pid(self, *args, **kwargs):
        """Gets the current pid of the running job"""
        status = self.get_status(*args, **kwargs)
        if "start/" not in status:
            raise ValueError(
                '{} is not in the running state.'.format(self.name))
        pid = status.split()[-1]
        if not pid.isdigit():
            raise ValueError(
                '{} does not have valid process id: {}'.format(self.name, pid))
        return int(pid)

    def is_running(self, *args, **kwargs):
        """Return True if the job is running. Otherwise, False."""
        return 'start/' in self.get_status(*args, **kwargs)

    def wait_for_job_spawned(self, timeout=10):
        """Wait until the named job is in spawned state."""
        spawned = wait_until(
            lambda: 'start/spawned' in self.get_status(), timeout=timeout)
        if not spawned:
            raise RuntimeError(
                'Job {} not in start/spwaned state within timeout {}'.format(
                    self.name, timeout))

    def ensure_stopped(self, timeout=300, *args, **kwargs):
        """Stops the job if it is running and then checks
        that it effectively was stopped
        """
        if self.is_running(*args, **kwargs):
            self.stop()
        if not wait_until(lambda: not self.is_running(), timeout=timeout):
            raise RuntimeError(
                'Job {} not stopped within timeout {}'.format(
                    self.name, timeout))

    @staticmethod
    def _run_command(command, sudo=False, *args, **kwargs):
        """Run the provided command
        :param command: The command to run
        :param sudo: True if you want to run the command with sudo privileges
        :param args: List with the arguments for the command
        :return: Output of command.
        """
        kwargs['universal_newlines'] = kwargs.get('universal_newlines', True)
        if sudo:
            return run_command_with_sudo(command, *args, **kwargs)
        else:
            command = '{c} {a}'.format(c=command, a=' '.join(args)).strip()
            return subprocess.check_output(command, shell=True,
                                           stderr=subprocess.STDOUT, **kwargs)
