# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import os
import signal
import subprocess

import psutil
from retrying import retry

from ubuntu_system_tests.helpers import (
    try_import
)

process_helpers = try_import('unity8.process_helpers')

PROCESS_STATUS_WAITING = 'S'
PROCESS_STATUS_RUNNING = 'R'
PRCOESS_STATUS_UNINTERRUPTABLE = 'D'
PROCESS_STATUS_STOPPED = 'T'
PROCESS_STATUS_NOT_RUNNING = 'X'
PROCESS_STATUS_ZOMBIE = 'Z'

PROCESS_RUNNING_STATES = [
    PROCESS_STATUS_WAITING,
    PROCESS_STATUS_RUNNING,
    PRCOESS_STATUS_UNINTERRUPTABLE,
    PROCESS_STATUS_STOPPED,
]

logger = logging.getLogger(__name__)

app_ids = None


class PidNotFoundError(RuntimeError):
    """Pid not found error."""
    def __init__(self, process_name, *args, **kwargs):
        message = 'No running process found for {}'.format(process_name)
        super().__init__(message, *args, **kwargs)


def is_process_running(pname):
    try:
        return bool(get_process_id(pname))
    except PidNotFoundError:
        return False


@retry(stop_max_attempt_number=5,
       wait_fixed=1000,
       retry_on_exception=lambda exception: isinstance(
           exception, PidNotFoundError))
def get_process_id(pname):
    """
    Return PID for the requested process.

    :param pname: name of the process to get PID of.
    """
    try:
        pids = subprocess.check_output(
            ['pidof', pname], universal_newlines=True).split()
    except subprocess.CalledProcessError:
        raise PidNotFoundError(pname)
    for pid in pids:
        if get_process_status(pid) in PROCESS_RUNNING_STATES:
            return int(pid)
    raise PidNotFoundError(pname)


def get_process_status(pid):
    """Return the status code for specifiec process id."""
    try:
        output = subprocess.check_output(['ps', '-p', pid, '-o', 'state'])
    except subprocess.CalledProcessError:
        return PROCESS_STATUS_NOT_RUNNING
    else:
        return output.decode().split()[1]


def stop_process(pname):
    if is_process_running(pname):
        [process.terminate() for process in psutil.process_iter() if
         process.name() == pname]


def start_application(package, *args):
    """Start application with specified package."""
    app_id = get_application_id(package)
    subprocess.check_call(['ubuntu-app-launch', app_id] + list(args))


def stop_application(package):
    """Stop application with specified package."""
    app_id = get_application_id(package)
    if app_id == package:
        # This is a native application.
        # Due to https://launchpad.net/bugs/1626028 use os.kill instead of
        # ubuntu-app-stop.
        try:
            pid = get_application_pid(package)
        except PidNotFoundError:
            pass  # app is already stopped
        else:
            os.kill(pid, signal.SIGKILL)
    else:
        # This is a web-app or click-app which works with ubuntu-app-stop
        subprocess.check_call(['ubuntu-app-stop', app_id])


def stop_all_applications():
    """Stop all opened applications."""
    open_apps = list_open_applications()
    for app in open_apps:
        stop_application(app)


def get_app_id_list():
    """Get list of installed app ids, using saved list if present."""
    global app_ids
    if not app_ids:
        app_ids = subprocess.check_output(
            ['ubuntu-app-launch-appids'],
            universal_newlines=True).strip().split()
    return app_ids


def list_open_applications():
    """Return a list of open application ids."""
    apps = subprocess.check_output(['ubuntu-app-list'],
                                   universal_newlines=True)
    return apps.split()


def is_application_running(package):
    """Return boolean indicating if app is currently running."""
    return get_application_id(package) in list_open_applications()


def get_application_id(package):
    """Return application id for specified app_id or package name."""
    if package in get_app_id_list():
        # The specified package is already a valid app id
        return package
    output = subprocess.check_output(
        ['ubuntu-app-triplet', package], universal_newlines=True).strip()
    if output:
        return output
    else:
        raise ValueError('Package {} not found'.format(package))


def get_application_pid(package):
    app_id = get_application_id(package)
    try:
        pid = int(subprocess.check_output(['ubuntu-app-pid', app_id],
                                          universal_newlines=True).strip())
    except (subprocess.CalledProcessError, ValueError):
        raise PidNotFoundError(app_id)
    return pid


def get_process_args(pid):
    """ Retrieve the command line used to start the process """
    return subprocess.check_output('xargs -0 < /proc/{}/cmdline'.format(pid),
                                   shell=True, universal_newlines=True)


def is_testability_enabled_for_process(pid):
    """Return True if testability is enabled for specified process."""
    proc_env = '/proc/{pid}/environ'.format(pid=pid)
    command = ['cat', proc_env]
    try:
        output = subprocess.check_output(
            command,
            stderr=subprocess.STDOUT,
            universal_newlines=True,
        )
    except subprocess.CalledProcessError as e:
        e.args += ('Failed to get environment for pid {}: {}.'.format(
            pid, e.output),)
        raise
    return output.find('QT_LOAD_TESTABILITY=1') >= 0
