# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import logging

import fixtures
import psutil
from dbus import DBusException
from retrying import retry

from ubuntu_system_tests.helpers import try_import
from ubuntu_system_tests.helpers.processes import (
    PidNotFoundError,
    is_testability_enabled_for_process,
    stop_all_applications,
)
from ubuntu_system_tests.helpers.unity8 import Unity8Job
from autopilot import (
    exceptions
)
fixture_setup = try_import('ubuntuuitoolkit.fixture_setup')
logger = logging.getLogger(__name__)


class SettingsWizard(fixtures.Fixture):

    WIZARD_FILE = '/home/phablet/.config/ubuntu-system-settings/wizard-has-run'

    def __init__(self, enable):
        super().__init__()
        self.enable = enable

    def setUp(self):
        super().setUp()
        original_state = self.is_settings_wizard_enabled()
        self.addCleanup(self.set_settings_wizard, original_state)
        if self.enable != original_state:
            self.set_settings_wizard(self.enable)

    def is_settings_wizard_enabled(self):
        return not os.path.exists(self.WIZARD_FILE)

    def set_settings_wizard(self, enabled):
        if enabled:
            self.enable_settings_wizard()
        else:
            self.disable_settings_wizard()

    def enable_settings_wizard(self):
        if os.path.exists(self.WIZARD_FILE):
            os.remove(self.WIZARD_FILE)

    def disable_settings_wizard(self):
        if not os.path.exists(self.WIZARD_FILE):
            open(self.WIZARD_FILE, 'a').close()


class StartUnity8(fixtures.Fixture):

    def __init__(self):
        self.unity8 = Unity8Job()

    def setUp(self, restart_unity=True):
        super().setUp()
        self.setup_unity(restart_unity=restart_unity)

    # Retry added until fix for bug 1427946 in QtMir is landed
    @retry(stop_max_attempt_number=5,
           wait_fixed=2000,
           retry_on_exception=lambda exception: (
               isinstance(exception, psutil.NoSuchProcess) or
               isinstance(exception, ValueError) or
               isinstance(exception, DBusException) or
               isinstance(exception, TimeoutError) or
               isinstance(exception, PidNotFoundError) or
               isinstance(exception, exceptions.ProcessSearchError) or
               isinstance(exception, exceptions.BackendException)))
    def setup_unity(self, restart_unity=True):
        """Setup unity8 environment, restarting if any error is observed.
        :param restart_unity: Whether unity8 should be restarted.
        """
        if restart_unity:
            self.unity8.restart()
        else:
            # If unity8 is not restarted, we need to make sure
            # that testability is enabled for the unity8 instance
            # which is already running, if it's not, then we have
            # no chance other than to restart unity8
            if not is_testability_enabled_for_process(
                    self.unity8.get_pid()):
                self.unity8.restart()
        self.addCleanup(stop_all_applications)


class EnableUnity8FakeSensors(fixtures.Fixture):
    """Fixture to enable fake sensors for unity8."""
    def __init__(self):
        self.unity8 = Unity8Job()

    def setUp(self):
        super().setUp()
        self.unity8.restart(fake_sensors=True)
