# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from subprocess import CalledProcessError
from threading import Event

from ubuntu_system_tests.helpers import ofono
from ubuntu_system_tests.helpers import try_import
from ubuntu_system_tests.helpers.notifications import sim_pin_observer
from ubuntu_system_tests.helpers.unity8.sensors_utils import Unity8Sensors
from ubuntu_system_tests.helpers import context
from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.helpers.autopilot import get_proxy_object
from ubuntu_system_tests.helpers.job import JobBase
from ubuntu_system_tests.helpers.unity8 import get_dash
from ubuntu_system_tests.helpers.unity8 import greeter

environment = try_import('ubuntuuitoolkit.environment')


class Unity8Job(JobBase):

    def __init__(self):
        super().__init__('unity8')
        self.sensors = Unity8Sensors()

    def ensure_unity_running_and_greeter_hidden(self):
        if not self.is_running():
            self.start()
            greeter.wait_for_greeter()
        greeter.hide_greeter_with_dbus()

    def start(self, fake_sensors=False, *args, **kwargs):
        """Start a new unity8 process.
        :param fake_sensors: If True then enable fake sensors,
        otherwise use real sensors.
        """
        api_test_override = 'UBUNTU_PLATFORM_API_TEST_OVERRIDE'
        try:
            environment.get_initctl_env_var(api_test_override)
        except CalledProcessError:
            pass
        else:
            environment.unset_initctl_env_var(api_test_override)
        if fake_sensors:
            environment.set_initctl_env_var(api_test_override, 'sensors')
            args = ['QT_LOAD_TESTABILITY=1', '--no-wait']
        JobBase._run_command(self.start_command, self.sudo, *args, **kwargs)
        if fake_sensors:
            self.sensors.validate_fifo_ready(self.name)
            self.sensors.setup_sensors(self.name)
        wait_until(self.is_running, timeout=60)
        context.shared.unity_pid = self.get_pid()
        unity = self.get_unity8_proxy_object()
        context.shared.unity = unity
        self.wait_for_unity_ready()
        unity.wait_for_greeter_content()
        unity.unlock()
        context.shared.dash = get_dash()

    def restart(self, fake_sensors=False):
        """Restart unity8 process, stopping existing process if running.
        :param fake_sensors: If True then enable fake sensors, otherwise use
        real sensors.
        """
        try:
            pid = self.get_pid()
        except CalledProcessError:
            pass
        else:
            JobBase._run_command('rm -f /tmp/sensor-fifo-{}'.format(pid),
                                 self.sudo)
            self.ensure_stopped()
        self.start(fake_sensors=fake_sensors)

    @staticmethod
    def wait_for_unity_ready(dismiss_sim_unlock=True):
        """Wait until unity is ready, dismissing sim pin entry notifications
        if required.
        :param dismiss_sim_unlock: Whether to dismiss sim pin screens.
        """
        if dismiss_sim_unlock:
            locked_sims = ofono.get_locked_sims()
            if locked_sims:
                # There are locked sims, so dismiss any pin entry screens.
                # The pin entry screen does not always show, depending on
                # whether the sims have been unlocked previously after boot.
                # So clear the screens from a different thread to avoid
                # blocking here if they don't show.
                completed = Event()
                pin_observer = sim_pin_observer.start_sim_pin_observer(
                    completed, locked_sims)
                # Wait until there are no notifications and greeter displayed.
                n = context.shared.unity.main_window.get_notifications_list()
                n.count.wait_for(0)
                context.shared.unity.wait_for_greeter_content()
                if not completed.is_set():
                    # There were no sim pin screens shown, but greeter is now
                    # ready, so stop the sim pin observer.
                    pin_observer.stop()
            else:
                context.shared.unity.wait_for_greeter_content()

    def get_unity8_proxy_object(self):
        from ubuntu_system_tests.helpers.unity8.shell import Unity8  # NOQA
        return get_proxy_object(self.get_pid())
